/****************************************************************************
 *   Copyright (C) 2009 Matteo Croce                                        *
 *   rootkit85@yahoo.it                                                     *
 ****************************************************************************/

#include <QFileDialog>
#include <QMessageBox>
#include <QFileInfo>
#include <zlib.h>

#include "warunpack.h"

WarUnPack::WarUnPack() : QMainWindow(0)
{
	setupUi(this);
	fileList->setVisible(false);
	QSize s = minimumSizeHint();
	s.setWidth(width());
	resize(s);
	connect(actionAbout, SIGNAL(triggered()), SLOT(about()));
	connect(browseIn, SIGNAL(clicked()), SLOT(browseInSlot()));
	connect(extract, SIGNAL(clicked()), SLOT(extractSlot()));
}

WarUnPack::~WarUnPack()
{
}

void WarUnPack::browseInSlot()
{
	QString path = QFileDialog::getOpenFileName(this, QString(), QString(),
			"MMG Archives (*.mmg)");
	if(path.length()) {
		fileList->clear();
		input->setText(path);
		path = path.remove(path.lastIndexOf(".mmg"), 4);
		if(path.length())
			output->setText(path);
	}
}

void WarUnPack::extractSlot()
{
	fileList->setVisible(true);

	QFile file(input->text());
	file.open(QIODevice::ReadOnly);

	char header[HEADER_SIZE];
	file.read(header, HEADER_SIZE);
	int numfiles = header[FILE_NUMBER] | header[FILE_NUMBER + 1] << 8;

	QList<mmg_index> files;
	for(int i = 0; i < numfiles; i++){
		char desc[INDEX_SIZE];
		file.read(desc, INDEX_SIZE);
		files += mmg_index(QString(desc), 
				(desc[LEN_OFFSET] & 0xff) | (desc[LEN_OFFSET + 1] & 0xff) << 8 | (desc[LEN_OFFSET + 2] & 0xff) << 16 | (desc[LEN_OFFSET + 3] & 0xff) << 24,
				(desc[UNC_LEN_OFFSET] & 0xff) | (desc[UNC_LEN_OFFSET + 1] & 0xff) << 8 | (desc[UNC_LEN_OFFSET + 2] & 0xff) << 16 | (desc[UNC_LEN_OFFSET + 3] & 0xff) << 24);
	}
	file.seek(file.pos() + 4);
	QDir().mkpath(output->text());
	for(int i = 0; i < files.size(); i++) {
		fileList->addItem(files[i].name);
		files[i].name.replace('\\', '/');
		char *comp = new char [files[i].len], *uncomp = new char[files[i].unclen];
		file.read(comp, files[i].len);
		uncompress((Bytef*)uncomp, &files[i].unclen, (Bytef*)comp, files[i].len);
		QDir().mkpath(output->text() + '/' + QFileInfo(files[i].name).path());
		QFile extr(output->text() + '/' + files[i].name);
		extr.open(QIODevice::WriteOnly);
		extr.write(uncomp, files[i].unclen);
		delete comp, uncomp;
	}
}

void WarUnPack::about()
{
	QMessageBox::about(this, "About WarUnPack", "WarUnPack<br><br>Brought to you by <a href=\"http://teknoraver.net/\">TeknoRaver</a>");
}

mmg_index::mmg_index(const QString &qs, int i, unsigned long u) : name(qs), len(i), unclen(u)
{
}
